=== modified file 'ivle/fileservice_lib/action.py'
--- ivle/fileservice_lib/action.py	2009-01-13 01:36:15 +0000
+++ ivle/fileservice_lib/action.py	2009-01-19 22:56:46 +0000
@@ -263,6 +263,35 @@
     except shutil.Error:
         raise ActionError("Could not move the file specified")
 
+def svn_movefile(req, frompath, topath, copy=False):
+    """Performs an svn move, resolving filenames, checking for any errors,
+    and throwing ActionErrors if necessary. Can also be used to do a copy
+    operation instead.
+
+    frompath and topath are straight paths from the client. Will be checked.
+    """
+    if frompath is None or topath is None:
+        raise ActionError("Required field missing")
+    frompath = actionpath_to_local(req, frompath)
+    topath = actionpath_to_local(req, topath)
+    if not os.path.exists(frompath):
+        raise ActionError("The source file does not exist")
+    if os.path.exists(topath):
+        if frompath == topath:
+            raise ActionError("Source and destination are the same")
+        raise ActionError("A file already exists with that name")
+
+    try:
+        if copy:
+            svnclient.copy(frompath, topath)
+        else:
+            svnclient.move(frompath, topath)
+    except OSError:
+        raise ActionError("Could not move the file specified")
+    except pysvn.ClientError:
+        raise ActionError("Could not move the file specified")  
+
+
 ### ACTIONS ###
 
 def action_delete(req, fields):
@@ -444,12 +473,7 @@
     files = fields.getlist('file')
     if dst is None or src is None or mode is None:
         raise ActionError("Required field missing")
-    if mode == "copy":
-        copy = True
-    elif mode == "move":
-        copy = False
-    else:
-        raise ActionError("Invalid mode (must be 'copy' or 'move')")
+
     dst_local = actionpath_to_local(req, dst)
     if not os.path.isdir(dst_local):
         raise ActionError("dst is not a directory")
@@ -464,7 +488,16 @@
         # The destination is found by taking just the basename of the file
         topath = os.path.join(dst, os.path.basename(file))
         try:
-            movefile(req, frompath, topath, copy)
+            if mode == "copy":
+                movefile(req, frompath, topath, True)
+            elif mode == "move":
+                movefile(req, frompath, topath, False)
+            elif mode == "svncopy":
+                svn_movefile(req, frompath, topath, True)
+            elif mode == "svnmove":
+                svn_movefile(req, frompath, topath, False)
+            else:
+                raise ActionError("Invalid mode (must be '(svn)copy' or '(svn)move')")
         except ActionError, message:
             # Store the error for later; we want to copy as many as possible
             if errormsg is None:
@@ -709,6 +742,7 @@
             raise util.IVLEError(404, 'The specified repository path does not exist')
         else:
             raise ActionError(str(e[0]))
+            
 
 # Table of all action functions #
 # Each function has the interface f(req, fields).

=== modified file 'ivle/fileservice_lib/listing.py'
--- ivle/fileservice_lib/listing.py	2009-01-13 01:36:15 +0000
+++ ivle/fileservice_lib/listing.py	2009-01-19 22:56:46 +0000
@@ -237,6 +237,11 @@
         try:
             for filename in os.listdir(path):
                 listing[filename.decode('utf-8')] = file_to_fileinfo(path, filename)[1]
+                try:
+                    svnclient.status(os.path.join(path, filename), recurse = False)
+                    listing[filename.decode('utf-8')]['svnstatus'] = 'normal'
+                except:
+                    pass
         except OSError:
             # Non-directories will error - that's OK, we just want the "."
             pass

=== modified file 'www/apps/browser/__init__.py'
--- www/apps/browser/__init__.py	2009-01-13 01:36:15 +0000
+++ www/apps/browser/__init__.py	2009-01-19 22:56:46 +0000
@@ -174,14 +174,16 @@
         ('upload',  ['Upload File',      'Upload a file to the current directory'])
       ]),
       ('Subversion', True, [
-        ('svnadd',      ['Add',          'Schedule the selected temporary files to be added permanently']),
-        ('svnremove',   ['Remove',       'Schedule the selected permanent files to be removed']),
-        ('svndiff',     ['Diff',         'View any changes to the selected file since its last committed state']),
-        ('svnrevert',   ['Revert',       'Restore the selected files back to their last committed state']),
-        ('svnupdate',   ['Update',       'Update your files with changes from the permanent repository']),
-        ('svncommit',   ['Commit',       'Commit any changes to the permanent repository']),
-        ('svnresolved', ['Mark Resolved','Mark a conflicted file as being resolved']),
-        ('svnlog',      ['View Log',     'View the log of commits of the selected file']),
+        ('svncut',      ['Svn Cut',      'Prepare to move the selected files to another directory, maintaining history']),
+        ('svncopy',     ['Svn Copy',     'Prepare to copy the selected files to another directory, maintaining history']),
+        ('svnadd',      ['Add',            'Schedule the selected temporary files to be added permanently']),
+        ('svnremove',   ['Remove',         'Schedule the selected permanent files to be removed']),
+        ('svndiff',     ['Diff',           'View any changes to the selected file since its last committed state']),
+        ('svnrevert',   ['Revert',         'Restore the selected files back to their last committed state']),
+        ('svnupdate',   ['Update',         'Update your files with changes from the permanent repository']),
+        ('svncommit',   ['Commit',         'Commit any changes to the permanent repository']),
+        ('svnresolved', ['Mark Resolved',  'Mark a conflicted file as being resolved']),
+        ('svnlog',      ['View Log',       'View the log of commits of the selected file']),
       ])
     ]
 

=== modified file 'www/media/browser/browser.js'
--- www/media/browser/browser.js	2008-08-05 02:06:49 +0000
+++ www/media/browser/browser.js	2009-01-19 22:56:46 +0000
@@ -545,10 +545,24 @@
     }
 }
 
+/* Updates the list of available actions based on files selected */
 function update_actions()
 {
     var file;
     var numsel = selected_files.length;
+    var svn_selection = false;
+    
+    if (numsel > 0)
+    {
+        svn_selection = true;
+        for (var i = 0; i < selected_files.length; i++){
+            if (file_listing[selected_files[i]]["svnstatus"] == "unversioned")
+            {
+                svn_selection = false;        
+            }
+        }
+    }
+    
     if (numsel <= 1)
     {
         if (numsel == 0)
@@ -619,8 +633,8 @@
      */
     var run = document.getElementById("act_run");
      
-    if (!file.isdir && file.type == "text/x-python" && numsel <= 1
-        && current_file.svnstatus != 'revision')
+    if (numsel <= 1 && !file.isdir && file.type == "text/x-python" 
+            && current_file.svnstatus != 'revision')
     {
         if (numsel == 0)
         {
@@ -703,7 +717,6 @@
     var pubcond = numsel <= 1 && file.isdir;
     if (pubcond)
     {
-        /* TODO: Work out of file is svn'd */
         /* If this dir is already published, call it "Unpublish" */
         if (file.published)
         {
@@ -742,8 +755,11 @@
     /* Subversion actions */
     /* These are only useful if we are in a versioned directory and have some
      * files selected. */
-    set_action_state(["svnadd", "svnremove", "svnrevert", "svncommit"], numsel >= 1 && current_file.svnstatus);
-
+    set_action_state(["svnadd",], numsel >= 1 && current_file.svnstatus);
+    /* And these are only usefull is ALL the selected files are versioned */
+    set_action_state(["svnremove", "svnrevert", "svncommit", "svncopy", 
+            "svncut"], numsel >= 1 && current_file.svnstatus && svn_selection);
+    
     /* Diff, log and update only support one path at the moment, so we must
      * have 0 or 1 versioned files selected. If 0, the directory must be
      * versioned. */
@@ -869,6 +885,12 @@
     case "svnlog":
         window.location = path_join(app_path('svnlog'), current_path, selected_files[0] || '');
         break;
+    case "svncopy":
+        action_svncopy(selected_files);
+        break;
+    case "svncut":
+        action_svncut(selected_files);
+        break;
     }
 }
 

=== modified file 'www/media/browser/listing.js'
--- www/media/browser/listing.js	2008-12-20 03:46:56 +0000
+++ www/media/browser/listing.js	2009-01-19 22:56:46 +0000
@@ -145,11 +145,26 @@
 {
     /* Get the "clipboard" object from the browser cookie */
     var clip_obj = read_cookie("clipboard");
+    var under_subversion;
+    
     if (clip_obj == null)
     {
         alert("No files have been cut or copied.");
         return false;
     }
+
+    if (clip_obj.mode == "svnmove" || clip_obj.mode == "svncopy")
+    {
+        under_subversion = ("svnstatus" in current_file) && 
+                                (current_file.svnstatus != "unversioned");
+        if (!under_subversion)
+        {
+            alert("Cannot perform an Subversion Move outside of"
+                                                + " Permanent directories!");
+            return false;
+        }        
+    }
+    
     /* The clip_obj is exactly what we want to pass, plus the current path
      * as destination. */
     clip_obj.dst = ".";
@@ -157,6 +172,18 @@
     return false;
 }
 
+function action_svncopy(files)
+{
+    action_copy_or_cut(files, "svncopy");
+    return false;
+}
+
+function action_svncut(files)
+{
+    action_copy_or_cut(files, "svnmove");
+    return false;
+}
+
 function action_add(files)
 {
     do_action("svnadd", current_path, {"path":files});
@@ -689,13 +716,10 @@
         row.appendChild(td);
         td = document.createElement("td");
         td.setAttribute("class", "thincol");
-        if (under_subversion)
-        {
-            var icon = svnstatus_to_icon(file.svnstatus);
-            if (icon)
-                td.appendChild(dom_make_img(icon, icon_size, icon_size,
-                    svnstatus_to_string(file.svnstatus)));
-        }
+        var icon = svnstatus_to_icon(file.svnstatus);
+        if (icon)
+            td.appendChild(dom_make_img(icon, icon_size, icon_size,
+                                        svnstatus_to_string(file.svnstatus)));
         row.appendChild(td);
 
         /* Column 3: Filename */

